@@ -1,4 +1,3 @@
-import logging
 
 class PyVException(Exception):
     pass
@@ -43,6 +42,9 @@ class MissingContractIdException(PyVException):
 class MissingTokenIdException(PyVException):
     pass
 
+class InvalidContractException(PyVException):
+    pass
+
 
 def set_throw_on_error(throw=True):
     global THROW_EXCEPTION_ON_ERROR
@@ -50,6 +52,6 @@ def set_throw_on_error(throw=True):
 
 
 def throw_error(msg, exception=PyVException):
-    logging.error(msg)
+    global THROW_EXCEPTION_ON_ERROR
     if THROW_EXCEPTION_ON_ERROR:
         raise exception(msg)
